/* eslint-disable no-undef */
import styled from '@emotion/styled';
import { ContentCopy, ContentPaste, DeleteForever } from '@mui/icons-material';
import AutoFixHighIcon from '@mui/icons-material/AutoFixHigh';
import {
  Divider,
  IconButton,
  ListItemIcon,
  ListItemText,
  Menu,
  MenuItem,
} from '@mui/material';
import { useState } from 'react';
import notification from './components/notification';

const Z_INDEX_MAX = 2 ** 31 - 1;

const CustomButton = styled(IconButton)`
  position: fixed;
  right: 20px;
  bottom: 20px;
  z-index: ${Z_INDEX_MAX - 1};
  opacity: 0.75;
  transition: all 225ms linear;
  background-color: #c2e0ff;
  &:hover {
    opacity: 1;
    background-color: #99ccf3;
  }
`;

const CustomMenu = styled(Menu)`
  z-index: ${Z_INDEX_MAX};
`;

const App = () => {
  const [anchorEl, setAnchorEl] = useState<null | HTMLElement>(null);
  const menuOpen = Boolean(anchorEl);
  const handleMenuOpen = (event: React.MouseEvent<HTMLButtonElement>) => {
    setAnchorEl(event.currentTarget);
  };
  const handleMenuClose = () => {
    setAnchorEl(null);
  };

  const copyToClipboard = (text: string) => {
    GM_setClipboard(text);
  };

  const getCookie = () =>
    new Promise((resolve, reject) => {
      GM_cookie('list', {}, (cookie, error) => {
        if (error) {
          reject(error);
        } else {
          resolve(cookie);
        }
      });
    });

  const setCookie = (cookieObj: TampermonkeyBeta.Cookie) =>
    new Promise((resolve, reject) => {
      GM_cookie('set', cookieObj, (cookie, error) => {
        if (error) {
          reject(error);
        } else {
          resolve(cookie);
        }
      });
    });

  const deleteCookie = async () => {
    const currentCookie = (await getCookie()) as TampermonkeyBeta.Cookie[];
    return new Promise((resolve, reject) => {
      currentCookie.forEach((cookieRecord) => {
        GM_cookie('delete', { name: cookieRecord.name }, (cookie, error) => {
          if (error) {
            reject(error);
          } else {
            resolve(cookie);
          }
        });
      });
    });
  };

  const getSession = async () => {
    try {
      const cookie = await getCookie();
      const exportSessionData = JSON.stringify(cookie);
      copyToClipboard(exportSessionData);
      notification.success({ message: 'Session 数据已复制到剪贴板' });
    } catch (error) {
      notification.error({ message: '获取 Cookie 失败' });
    }
  };

  const setSession = () => {
    try {
      const importSessionData: string =
        prompt('在此处粘贴你的 Session 数据', '[{}]') || '[{}]';
      const cookie = JSON.parse(importSessionData) as TampermonkeyBeta.Cookie[];
      cookie.forEach(async (item) => {
        await setCookie(item);
      });
      notification.success({ message: 'Session 数据已导入' });
    } catch (error) {
      notification.error({ message: '导入 Cookie 失败' });
    }
  };

  const deleteSession = async () => {
    try {
      await deleteCookie();
      notification.success({ message: 'Session 数据已删除' });
    } catch (error) {
      notification.error({ message: '删除 Cookie 失败' });
    }
  };

  const handleGetSession = () => {
    getSession();
    handleMenuClose();
  };

  const handleSetSession = () => {
    setSession();
    handleMenuClose();
  };

  const handleDeleteSession = () => {
    deleteSession();
    handleMenuClose();
  };

  return (
    <div>
      <CustomButton
        color="primary"
        id="custom-button"
        aria-controls={menuOpen ? 'custom-menu' : undefined}
        aria-haspopup="true"
        aria-expanded={menuOpen ? 'true' : undefined}
        onClick={handleMenuOpen}
      >
        <AutoFixHighIcon />
      </CustomButton>

      <CustomMenu
        id="custom-menu"
        anchorEl={anchorEl}
        open={menuOpen}
        onClose={handleMenuClose}
        MenuListProps={{
          'aria-labelledby': 'custom-button',
        }}
      >
        <MenuItem onClick={handleGetSession}>
          <ListItemIcon>
            <ContentCopy fontSize="small" />
          </ListItemIcon>
          <ListItemText>导出 Session 数据</ListItemText>
        </MenuItem>

        <MenuItem onClick={handleSetSession}>
          <ListItemIcon>
            <ContentPaste fontSize="small" />
          </ListItemIcon>
          <ListItemText>导入 Session 数据</ListItemText>
        </MenuItem>

        <Divider />

        <MenuItem onClick={handleDeleteSession}>
          <ListItemIcon>
            <DeleteForever fontSize="small" />
          </ListItemIcon>
          <ListItemText>清除 Session 数据</ListItemText>
        </MenuItem>
      </CustomMenu>
    </div>
  );
};

export default App;
